//  ColorHCFR
//  HCFRSensor.h
// -----------------------------------------------------------------------------
//  Created by Jrme Duquennoy on 31/08/07.
//
//  $Rev: 51 $
//  $Date: 2008-05-22 14:48:37 +0200 (Thu, 22 May 2008) $
//  $Author: jerome $
// -----------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation; either version 2 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
// the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
// Public License for more details.
//
// You should have received a copy of the GNU General Public License along with this program; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// -----------------------------------------------------------------------------


#import <Cocoa/Cocoa.h>
#import <HCFRPlugins/HCFRDataSource.h>

#define kHCFRSensorFailureException @"HCFRSensorFailurException"

/*!
    @class
    @abstract    Interface d'accs aux capteurs
    @discussion  Cette classe permet d'accder aux capteurs.
 C'est une couche d'abstraction qui uniformise l'accs aux donnes des capteurs.
 Une seul instance sera cre lors de l'execution. Si plusieurs sondes sont connectes,
 l'instance devra proposer  l'utilisateur de choisir la sonde  utiliser.
*/
@interface HCFRSensor : NSObject <HCFRDataSource> {
  Matrix  sensorToXYZMatrix;
  BOOL    calibrationPerformed;

  BOOL    waitForInteraction; // utilis pour attendre une interaction utilisateur.
  int     lastAlertReturnCode;
}
#pragma mark constructeur et desturcteur
/*!
    @function 
    @abstract   Initialisation
    @discussion Cette fonction sera appele lors de la cration de l'objet
 Il sera trs probablement necessaire de redfinir cette fonction pour chaque sous classe
 afin de s'enregistrer au niveau de l'OS pour tre averti des connexion/dconnexion
 des sondes USB.
    @result     L'objet initialis
*/
-(HCFRSensor*) init;
  /*!
  @function 
   @abstract   Desctrucion
   @discussion Cette fonction sera appele lors de la destruction de l'objet
   Il sera trs probablement necessaire de redfinir cette fonction pour chaque sous classe
   afin de se ds-inscrire pour l'coute des evenements USB.
   @result     L'objet initialis
   */
-(void) dealloc;

#pragma mark fonction divers
/*!
    @function 
    @abstract   Le nom du capteur
    @discussion Cette fonction retourne le nom du capteur pris en charge
    @result     Une chaine de caractres autorelease.
*/
-(NSString*) sensorName;
/*!
    @function 
    @abstract   Fonction appele lorsque les paramtres du capteur sont affichs
    @discussion Cette fonction est prvue pour donner  la classe un oportunit de mettre  jour la liste
 des capteurs connects avant affichage des paramtres.
 Elle sera appele  chaque fois que le panneau de configuration sera affich.
*/
-(void) refresh;

#pragma mark Gestion du capteur
/*!
    @function 
    @abstract   Le capteur est-il utilisable ?
    @discussion Cette fonction permet au programme de savoir si le capteur reprsent par la classe
 est utilisable (connect, et en tat de fonctionnement).
 Doit tre surcharge pour dfinire un nouveau capteur.
    @result     YES si le capteur est utilisable, NO sinon.
*/
-(BOOL) isAvailable;
/*!
    @function 
    @abstract   Le capteur doit-il tre calibr avant utilisation ?
    @discussion Cette fonction indique si le capteur ncssite une calibration avant utilisation ou non.
 Doit tre surcharge pour dfinire un nouveau capteur.
    @result     Un boolean indiquant si une calibration est requise.
*/
-(BOOL) needsCalibration;
/*!
    @function 
    @abstract   Les donnes de calibration sont elles valides ?
    @discussion Cette fonction indique si les donnes de calibration sont valides ou non.
    Si le capteur n'a pas besoin de calibration, cette fonction retournera toujours YES.
    Si cette fonction retourne NO, il faudra charger des donnes de calibration avant d'effectuer une mesure.
    @result     Un boolean indiquant si les donnes de calibration sont valides.
*/
-(BOOL) calibrationPerformed;
  /*!
    @function 
    @abstract   Met  jour la matrice de calibration du capteur.
    @discussion Cette fonction permet de mettre  jour la matrice de calibration du capteur.
 Cette matrice permet d'obtenir une couleur XYZ avec les donnes du capteur.
 Cette fonction n'a d'intret que pour les capteurs necessitant une calibration !
    @param      newCalibrationMatrix La nouvelle matrice de calibration
*/
-(void) setSensorToXYZMatrix:(Matrix)newSensorToXYZMatrix;
/*!
    @function 
    @abstract   Retourne la matrice de calibration actuelle du capteur.
    @discussion Cette fonction n'a d'intret que si le capteur necessite une calibration !
 Ne doit pas tre surcharge.
    @result     La matrice de calibration.
*/
-(Matrix) getSensorToXYZMatrix;

#pragma mark Gestion du fichier de calibration
/*!
   @function 
   @abstract   Charge les donnes d'un fichier de calibration HCFR.
   @discussion Cette fonction n'a d'intret que si le capteur necessite une calibration.
   @param      filePath Le chemin du fichier  charger
   @result     YES si le fichier  t charg, NO si une erreur est survenue.
*/
-(BOOL) readCalibrationFile:(NSString *)filePath;

#pragma mark Gestion du panneau de configuration
/*!
@function 
 @abstract   Le capteur a-t-il un panneau de configuration ?
 @discussion Si le capteur dispose d'un panneau de configuration, surchargez
 cette methode pour qu'elle retourne YES. La methode showSetupPanel devra tre dfinie
 Doit tre surcharge pour dfinire un nouveau capteur.
 */
-(BOOL) hasASetupView;
  /*!
  @function 
   @abstract   Retourne la vue de configuration du capteur
   @discussion Cette methode doit tre surcharge pour charger et retourner la vue
   de configuration du capteur. Elle ne sera appele que si hasASetupView retourne YES.
   Par defaut, cette fonction ne fait rien, elle doit tre surcharge pour dfinire un nouveau capteur.
   */
-(NSView*) setupView;

#pragma mark implementation du protocol DataSource
/*!
   @function 
   @abstract   Indique la prochaine valeur qui sera demande
   @discussion Cette fonction n'a rellement d'intrt que pour les capteurs virtuels, qui doivent
   retourner une valeur sans qu'il n'y ai de mesure effectue. Pour les autres capteurs, il est innnutile
   de la redfinir.
   */
-(void) nextMeasureWillBeType:(HCFRDataType)type referenceValue:(NSNumber*)referenceValue;
  /*!
  @function 
   @abstract   Effectue une mesure, et retourne le rsultat
   @discussion Cette fonction charge une mesure du capteur, puis la retourne sous forme d'un objet
   HCFRColor autoreleas. Si il n'est pas possible de faire une mesure, ou si la mesure  choue
   (capteur dconnect par exemple), la fonction retourne nil.
   Doit tre surcharge pour dfinire un nouveau capteur.
   Cette methode doit lever une exception de nom kHCFRSensorFailureException si il est impossible de retourner la mesure.
   @result     Le rsultat de la mesure. L'objet est autoreleas. La fonction ne doit jamais retourner nil !
   */
-(HCFRColor*) currentMeasure;

#pragma mark les fonction d'alerte
  /*!
  @function 
   @abstract   Affiche une alerte et attend qu'elle soit valide
   @discussion Le gnrateur tourne dans une thread spare. Cette fonction permet d'afficher une alerte,
   et de bloquer la thread du gnrateur en attendant la rponse.
   @param      alert L'alert  afficher
   @result     le code de retour de l'alerte (NSAlert...Return)
   */
-(int) displayAlert:(NSAlert*) alert;
@end
